#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2021 gr-kalvantula author.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy as np
from gnuradio import gr
from scipy.stats import circmean

DEBUG = True

class Auto_calibrate_phases(gr.interp_block):
    """
    docstring for block Auto_calibrate_phases
    """
    def __init__(self, calibration_mode, freq, prop_delay_sla1, prop_delay_sla2, prop_delay_mast, interp):
        gr.interp_block.__init__(self,
            name="Auto_calibrate_phases",
            in_sig=[(np.complex64, interp), (np.complex64, interp), (np.complex64, interp), np.float32, np.float32],
            out_sig=[np.complex64, np.complex64, np.complex64],
            interp = interp)

        self.sla1_phase_fix = 0
        self.sla2_phase_fix = 0
        self.calibration_mode = calibration_mode
        self.calibration_sample_count = 0
        self.calibration_discard_sample_count = 1e5
        self.sla1_phase_avg = 0
        self.sla2_phase_avg = 0
        self.avg_started = False
        self.mast_phase_avg = 0
        self.freq = freq
        self.sla1_prop_fix = 2 * np.pi * freq * prop_delay_sla1
        self.sla2_prop_fix = 2 * np.pi * freq * prop_delay_sla2



    def work(self, input_items, output_items):
        sla1_in = input_items[0].flatten()
        sla2_in = input_items[1].flatten()
        mast_in = input_items[2].flatten()
        sla1_phase = input_items[3]
        sla2_phase = input_items[4]
        sla1_out = output_items[0]
        sla2_out = output_items[1]
        mast_out = output_items[2]

        if self.calibration_mode:
            if self.calibration_sample_count > self.calibration_discard_sample_count:
                sla1_phase_avg = circmean(sla1_phase)
                sla2_phase_avg = circmean(sla2_phase)
                if self.avg_started:
                    self.sla1_phase_avg = circmean(self.sla1_phase_avg, sla1_phase_avg)
                    self.sla2_phase_avg = circmean(self.sla2_phase_avg, sla2_phase_avg)
                    self.avg_started = True
                else:
                    self.sla1_phase_avg = sla1_phase_avg
                    self.sla2_phase_avg = sla2_phase_avg
            self.calibration_sample_count += sla1_in.size

        # <+signal processing here+>
        sla1_out[:] = sla1_in * np.exp(0-self.sla1_phase_fix*1j)
        sla2_out[:] = sla2_in * np.exp(0-self.sla2_phase_fix*1j)
        mast_out[:] = mast_in
        return len(sla1_in)

    def set_calibration_mode(self, calibration_mode):
        if DEBUG:
            if calibration_mode:
                print("Starting calibration")
                self.calibration_sample_count = 0
            else:
                print("Calibration finished")
                self.sla1_phase_fix = self.sla1_phase_avg + self.sla1_prop_fix
                self.sla2_phase_fix = self.sla2_phase_avg + self.sla2_prop_fix
        self.calibration_mode = calibration_mode
