#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Kalle-Gustav Kruus.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy as np
from gnuradio import gr
from pyargus.directionEstimation import gen_ula_scanning_vectors, corr_matrix_estimate, DOA_plot
from pyargus.directionEstimation import DOA_Bartlett, DOA_Capon, DOA_LPM, DOA_MEM, DOA_MUSIC

incident_angles = np.arange(0, 360, 1)

class DOA_general_2ant(gr.sync_block):
    """
    sample_size := number of samples to calculate the DOA on
    element_spacing := spacing between antennas in wavelengths
    """
    def __init__(self, sample_size, element_spacing, method, update_every = 1, log_output=True):
        gr.sync_block.__init__(self,
            name="DOA general 2 ant",
            in_sig=[(np.complex64,sample_size), (np.complex64,sample_size)],
            out_sig=[(np.float32,incident_angles.size)],
            )
        array_alignment = np.arange(0, 2, 1) * element_spacing
        self.ula_scanning_vectors = gen_ula_scanning_vectors(array_alignment, incident_angles)
        self.sample_size = sample_size
        self.method = method
        self.log_output = log_output
        self.update_every = update_every
        self.buffered_result = np.zeros(360)
        self.counter = 0

    def work(self, input_items, output_items):
        for i in range(len(input_items[0])):
            self.counter += 1

            if(self.counter >= self.update_every):
                signal = np.zeros((2, self.sample_size), dtype=np.complex64)
                signal[0][:] = input_items[0][0]
                signal[1][:] = input_items[1][0]

                R = corr_matrix_estimate(signal.T)
                if self.method == "Bartlett":
                    DOA_data = DOA_Bartlett(R, self.ula_scanning_vectors)
                elif self.method == "Capon":
                    DOA_data = DOA_Capon(R, self.ula_scanning_vectors)
                elif self.method == "MEM":
                    DOA_data = DOA_MEM(R, self.ula_scanning_vectors, column_select = 1)
                elif self.method == "LPM":
                    DOA_data = DOA_LPM(R, self.ula_scanning_vectors, element_select = 0)
                else:
                    DOA_data = DOA_MUSIC(R, self.ula_scanning_vectors, signal_dimension = 1)
                
                if self.log_output:
                    DOA_data = np.divide(np.abs(DOA_data), np.max(np.abs(DOA_data)))  # normalization
                    log_scale_min = -50
                    DOA_data = 10 * np.log10(DOA_data)
                    theta_index = 0
                    for theta in incident_angles:
                        if DOA_data[theta_index] < log_scale_min:
                            DOA_data[theta_index] = log_scale_min
                        theta_index += 1
                self.buffered_result = DOA_data
                output_items[0][i][:] = DOA_data
                self.counter = 0
        #DOA_plot(DOA_data, incident_angles)
        #print(DOA_data.min(), DOA_data.max())
            else:
                output_items[0][i][:] = self.buffered_result
        return len(output_items[0])

    def set_element_spacing(self, new_element_spacing):
        array_alignment = np.arange(0, 2, 1) * new_element_spacing
        self.ula_scanning_vectors = gen_ula_scanning_vectors(array_alignment, incident_angles)
