#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Kalle-Gustav Kruus.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy as np
from gnuradio import gr


class FFT_phase_finder(gr.decim_block):  # other base classes are basic_block, decim_block, interp_block
    """Embedded Python Block example - a simple multiply const"""

    def __init__(self, fft_size, bin_idx, calibration_mode):  # only default arguments here
        """arguments to this function show up as parameters in GRC"""
        gr.decim_block.__init__(
            self,
            name='FFT find phase',   # will show up in GRC
            in_sig=[np.complex64],
            out_sig=[np.float32],
            decim=fft_size
        )
        self.calibration_mode = calibration_mode
        self.fft_size = fft_size
        self.bin_idx = bin_idx
        # if an attribute with the same name as a parameter is found,
        # a callback is registered (properties work, too).

    def work(self, input_items, output_items):
        """example: multiply with constant"""
        in_len = len(input_items[0])
        if in_len < self.fft_size:
            return 0
        elif not self.calibration_mode:
            output_items[0][:] = 0
            return in_len // self.fft_size
        else:
            for i in range(in_len // self.fft_size):
                fft_res = np.fft.fft(input_items[0][i*self.fft_size:(i+1)*self.fft_size])
                fft_phases = np.angle(fft_res)
                output_items[0][i] = fft_phases[self.bin_idx]
            return i

    def set_calibration_mode(self, calibration_mode):
        self.calibration_mode = calibration_mode

    def set_bin_idx(self, bin_idx):
        self.bin_idx = bin_idx
