#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Kalle-Gustav Kruus.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy as np
from gnuradio import gr
from scipy.stats import vonmises, circmean

class GEN_phase_diffs_to_ang(gr.decim_block):  # other base classes are basic_block, decim_block, interp_block

    def __init__(self, frequency, mast_x, mast_y, sla1_x, sla1_y, sla2_x, sla2_y, decim):  # only default arguments here
        gr.decim_block.__init__(
            self,
            name='Find angle from phase diffs',   # will show up in GRC
            in_sig=[np.float32, np.float32, np.float32],
            out_sig=[(np.float32, 360), (np.float32, 360), (np.float32, 360), (np.float32, 360)],
            decim=decim
        )
        self.far_dist = 2000
        self.wavelen = 299792458 / frequency
        self.sla1_mast_len, self.sla1_mast_ang = self.ant_dist_and_angle(sla1_x, sla1_y, mast_x, mast_y)
        self.sla2_mast_len, self.sla2_mast_ang = self.ant_dist_and_angle(sla2_x, sla2_y, mast_x, mast_y)
        self.sla1_sla2_len, self.sla1_sla2_ang = self.ant_dist_and_angle(sla1_x, sla1_y, sla2_x, sla2_y)
        print("ang", self.sla1_sla2_ang)
        self.range = np.linspace(-np.pi, np.pi, 360)
        self.decim = decim

        # if an attribute with the same name as a parameter is found,
        # a callback is registered (properties work, too).

    def ant_dist_and_angle(self, start_x, start_y, end_x, end_y):
        vec = np.complex(end_x - start_x, end_y - start_y)
        return np.abs(vec), np.angle(vec)

    def get_distribution_for_pair(self, ph_diff, ant_dist, ang_offset):
        dist = np.zeros(360)
        variant = 0
        while True:
            ph_diff_var = ph_diff - np.pi*2*variant if ph_diff <= np.pi else ph_diff - 2*np.pi + np.pi*2*variant
            dist_diff = ph_diff_var/(2*np.pi) * self.wavelen 
            if abs(dist_diff) > abs(ant_dist) * 1.1:
                break
            if dist_diff < -ant_dist:
                dist_diff = -ant_dist
            elif dist_diff > ant_dist:
                dist_diff = ant_dist
            #dist_diff = min(ant_dist, dist_diff)
            angle = np.arccos(dist_diff / ant_dist)
            dist += vonmises.pdf(self.range, 10, angle + ang_offset, 1)
            dist += vonmises.pdf(self.range, 10, -angle + ang_offset, 1)
            variant += 1

        return dist / max(1, np.max(dist))


    def work(self, input_items, output_items):
        sla1_sla2_diff_in = input_items[0]
        sla1_mast_diff_in = input_items[1]
        sla2_mast_diff_in = input_items[2]
        out0 = output_items[0]
        out1 = output_items[1]
        out2 = output_items[2]
        out3 = output_items[3]

        for i in range(len(out0)):
            med_sla1_sla2_diff = circmean(sla1_sla2_diff_in[i*self.decim : i*self.decim + self.decim])
            dist1 = self.get_distribution_for_pair(med_sla1_sla2_diff, self.sla1_sla2_len, self.sla1_sla2_ang)

            med_sla1_mast_diff = circmean(sla1_mast_diff_in[i*self.decim : i*self.decim + self.decim])
            dist2 = self.get_distribution_for_pair(med_sla1_mast_diff, self.sla1_mast_len, self.sla1_mast_ang)

            med_sla2_mast_diff = circmean(sla2_mast_diff_in[i*self.decim : i*self.decim + self.decim])
            dist3 = self.get_distribution_for_pair(med_sla2_mast_diff, self.sla2_mast_len, self.sla2_mast_ang)
            
            out0[i, :] = dist1 * dist2 * dist3
            out1[i, :] = dist1
            out2[i, :] = dist2
            out3[i, :] = dist3
          

        return len(out0)

