#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Kalle-Gustav Kruus.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy as np
from gnuradio import gr
from scipy.stats import vonmises, circmean


class ULA_phase_diffs_to_ang(gr.decim_block):  # other base classes are basic_block, decim_block, interp_block

    def __init__(self, frequency, ant_dist, decim):  # only default arguments here
        gr.decim_block.__init__(
            self,
            name='FFT find phase',   # will show up in GRC
            in_sig=[np.float32, np.float32, np.float32],
            out_sig=[(np.float32, 1024)],
            decim=decim
        )
        self.far_dist = 2000
        self.wavelen = 299792458 / frequency
        self.ant_dist = ant_dist
        self.range = np.linspace(-np.pi, np.pi, 1024)
        self.decim = decim


    def get_distribution_for_pair(self, ph_diff, ant_dist):
        dist = np.zeros(1024)
        variant = 0
        amount_adds = 0
        while True:
            #print("ph raw - ", ph_diff)
            ph_diff_var = ph_diff - np.pi*2*variant if ph_diff <= np.pi else ph_diff - 2*np.pi + np.pi*2*variant
            #print("ph fix - ", ph_diff_var)
            dist_diff = ph_diff_var/(2*np.pi) * self.wavelen + np.pi*2*variant
            #print("dist diff - ", dist_diff, "  Ant dist - ", ant_dist)
            if abs(dist_diff) > abs(ant_dist):
                #print("breaking")
                break
            angle = np.arccos(dist_diff / ant_dist)
            #print("angle")
            dist += vonmises.pdf(self.range, 5, angle, 1)
            dist += vonmises.pdf(self.range, 5, -angle, 1)
            amount_adds += 2
            variant += 1

        if amount_adds > 0:
            return dist/amount_adds
        return dist


    def work(self, input_items, output_items):
        in0 = input_items[0]
        in1 = input_items[1]
        in2 = input_items[2]
        out0 = output_items[0]

        for i in range(len(out0)):
            #print("1 - ", in0[0])
            med_in0 = circmean(in0[i*self.decim : i*self.decim + self.decim])
            dist1 = self.get_distribution_for_pair(med_in0, -self.ant_dist)
            #print("2- ", in1[0])
            med_in1 = circmean(in1[i*self.decim : i*self.decim + self.decim])
            dist2 = self.get_distribution_for_pair(med_in1, self.ant_dist)
            out0[i, :] = dist1 #* dist2
          

        return len(out0)

