#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Kalle-Gustav Kruus.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#


import numpy as np
from gnuradio import gr


class ULA_simulate_phase_diffs(gr.decim_block):  # other base classes are basic_block, decim_block, interp_block

    def __init__(self, frequency, angle, ant_dist):  # only default arguments here
        gr.decim_block.__init__(
            self,
            name='FFT find phase',   # will show up in GRC
            in_sig=[np.float32],
            out_sig=[np.float32, np.float32, np.float32],
            decim=1024
        )
        self.far_dist = 2000
        self.wavelen = 299792458 / frequency
        self.angle = np.radians(angle)
        self.ant_dist = ant_dist
        self.ph_sla2_mast = 0
        self.ph_sla1_mast = 0
        self.ph_sla1_sla2 = 0
        self.calc_phase_diffs()

        # if an attribute with the same name as a parameter is found,
        # a callback is registered (properties work, too).

    def calc_phase_diffs(self):
        sig_coord_x = 2000 * np.cos(self.angle)
        sig_coord_y = 2000 * np.sin(self.angle)

        dist_mast = np.sqrt(np.square(sig_coord_x - self.ant_dist) + np.square(sig_coord_y))
        dist_sla1 = np.sqrt(np.square(sig_coord_x) + np.square(sig_coord_y))
        dist_sla2 = np.sqrt(np.square(sig_coord_x + self.ant_dist) + np.square(sig_coord_y))

        diff_sla1_mast = dist_sla1 - dist_mast
        diff_sla2_mast = dist_sla2 - dist_mast
        diff_sla1_sla2 = dist_sla1 - dist_sla2

        self.ph_sla1_mast = diff_sla1_mast / self.wavelen * 2*np.pi
        self.ph_sla2_mast = diff_sla2_mast / self.wavelen * 2*np.pi
        self.ph_sla1_sla2 = diff_sla1_sla2 / self.wavelen * 2*np.pi

    def work(self, input_items, output_items):
        out0 = output_items[0]
        out1 = output_items[1]
        out2 = output_items[2]
        in0 = input_items[0]

        out0[:] = self.ph_sla1_mast            
        out1[:] = self.ph_sla2_mast
        out2[:] = self.ph_sla1_sla2     

        return len(out0)

    def set_angle(self, angle):
        self.angle = np.radians(angle)
        self.calc_phase_diffs()

